import React from 'react';
export interface TLPage<T extends TLShape> {
    id: string;
    shapes: Record<string, T>;
    bindings: Record<string, TLBinding>;
    backgroundColor?: string;
}
export interface TLPageState {
    id: string;
    brush?: TLBounds;
    pointedId?: string;
    hoveredId?: string;
    editingId?: string;
    editingBindingId?: string;
    boundsRotation?: number;
    currentParentId?: string;
    selectedIds: string[];
    camera: {
        point: number[];
        zoom: number;
    };
}
export interface TLHandle {
    id: string;
    index: number;
    point: number[];
}
export interface TLShape {
    id: string;
    type: string;
    parentId: string;
    childIndex: number;
    name: string;
    point: number[];
    rotation?: number;
    children?: string[];
    handles?: Record<string, TLHandle>;
    isLocked?: boolean;
    isHidden?: boolean;
    isEditing?: boolean;
    isGenerated?: boolean;
    isAspectRatioLocked?: boolean;
}
export declare type TLShapeUtils<T extends TLShape> = Record<string, TLShapeUtil<T>>;
export interface TLRenderInfo {
    ref?: React.RefObject<HTMLElement>;
    isEditing: boolean;
    isHovered: boolean;
    isSelected: boolean;
    isBinding: boolean;
    isDarkMode: boolean;
    isCurrentParent: boolean;
}
export interface TLTool {
    id: string;
    name: string;
}
export interface TLBinding {
    id: string;
    type: string;
    toId: string;
    fromId: string;
}
export interface TLSettings {
    isDebugMode: boolean;
    isDarkMode: boolean;
    isPenMode: boolean;
}
export interface TLTheme {
    brushFill?: string;
    brushStroke?: string;
    selectFill?: string;
    selectStroke?: string;
    background?: string;
    foreground?: string;
}
export interface TLCallbacks {
    onChange: (ids: string[]) => void;
    onPinchStart: (point: number[]) => void;
    onPinchEnd: (point: number[]) => void;
    onPinch: (info: TLPointerInfo & {
        distanceDelta: number;
    }) => void;
    onPan: (info: TLPointerInfo & {
        delta: number[];
    }) => void;
    onZoom: (info: TLPointerInfo & {
        delta: number;
    }) => void;
    onPointerMove: (info: TLPointerInfo) => void;
    onStopPointing: (info: TLPointerInfo) => void;
    onPointShape: (info: TLPointerInfo) => void;
    onDoublePointShape: (info: TLPointerInfo) => void;
    onRightPointShape: (info: TLPointerInfo) => void;
    onMoveOverShape: (info: TLPointerInfo) => void;
    onUnhoverShape: (info: TLPointerInfo) => void;
    onHoverShape: (info: TLPointerInfo) => void;
    onPointCanvas: (info: TLPointerInfo) => void;
    onDoublePointCanvas: (info: TLPointerInfo) => void;
    onRightPointCanvas: (info: TLPointerInfo) => void;
    onPointBounds: (info: TLPointerInfo) => void;
    onDoublePointBounds: (info: TLPointerInfo) => void;
    onRightPointBounds: (info: TLPointerInfo) => void;
    onPointBoundsHandle: (info: TLPointerInfo) => void;
    onDoublePointBoundsHandle: (info: TLPointerInfo) => void;
    onPointHandle: (info: TLPointerInfo) => void;
    onDoublePointHandle: (info: TLPointerInfo) => void;
    onRightPointHandle: (info: TLPointerInfo) => void;
    onMoveOverHandle: (info: TLPointerInfo) => void;
    onHoverHandle: (info: TLPointerInfo) => void;
    onUnhoverHandle: (info: TLPointerInfo) => void;
    onBlurEditingShape: () => void;
    onError: (error: Error) => void;
}
export interface TLBounds {
    minX: number;
    minY: number;
    maxX: number;
    maxY: number;
    width: number;
    height: number;
    rotation?: number;
}
export declare type TLIntersection = {
    didIntersect: boolean;
    message: string;
    points: number[][];
};
export declare enum TLBoundsEdge {
    Top = "top_edge",
    Right = "right_edge",
    Bottom = "bottom_edge",
    Left = "left_edge"
}
export declare enum TLBoundsCorner {
    TopLeft = "top_left_corner",
    TopRight = "top_right_corner",
    BottomRight = "bottom_right_corner",
    BottomLeft = "bottom_left_corner"
}
export interface TLPointerInfo {
    target: string;
    pointerId: number;
    origin: number[];
    point: number[];
    pressure: number;
    shiftKey: boolean;
    ctrlKey: boolean;
    metaKey: boolean;
    altKey: boolean;
}
export interface TLKeyboardInfo {
    origin: number[];
    point: number[];
    key: string;
    keys: string[];
    shiftKey: boolean;
    ctrlKey: boolean;
    metaKey: boolean;
    altKey: boolean;
}
export interface TLTransformInfo<T extends TLShape> {
    type: TLBoundsEdge | TLBoundsCorner;
    initialShape: T;
    scaleX: number;
    scaleY: number;
    transformOrigin: number[];
}
export interface TLBezierCurveSegment {
    start: number[];
    tangentStart: number[];
    normalStart: number[];
    pressureStart: number;
    end: number[];
    tangentEnd: number[];
    normalEnd: number[];
    pressureEnd: number;
}
export declare abstract class TLShapeUtil<T extends TLShape> {
    boundsCache: WeakMap<TLShape, TLBounds>;
    isEditableText: boolean;
    isAspectRatioLocked: boolean;
    canEdit: boolean;
    abstract type: T['type'];
    abstract defaultProps: T;
    abstract render(shape: T, info: TLRenderInfo): JSX.Element;
    abstract getBounds(shape: T): TLBounds;
    abstract getRotatedBounds(shape: T): TLBounds;
    abstract hitTest(shape: T, point: number[]): boolean;
    abstract hitTestBounds(shape: T, bounds: TLBounds): boolean;
    abstract transform(shape: T, bounds: TLBounds, info: TLTransformInfo<T>): TLShapeUtil<T>;
    abstract transformSingle(shape: T, bounds: TLBounds, info: TLTransformInfo<T>): TLShapeUtil<T>;
    create(props: Partial<T>): T;
    translateTo(shape: T, point: number[]): this;
    translateBy(shape: T, delta: number[]): this;
    rotateTo(shape: T, rotation: number): this;
    rotateBy(shape: T, rotation: number): this;
    mutate(shape: T, props: Partial<T>): this;
    shouldRender(prev: T, next: T): boolean;
    shouldDelete(shape: T): boolean;
    getCenter(shape: T): number[];
    setProperty<P extends keyof T>(shape: T, prop: P, value: T[P]): TLShapeUtil<T>;
    onChildrenChange(shape: T, children: TLShape[]): TLShapeUtil<T>;
    onBindingChange(shape: T, bindings: TLBinding, target: TLShape, targetBounds: TLBounds): TLShapeUtil<T>;
    onHandleChange(shape: T, handle: Partial<T['handles']>, info: TLPointerInfo): TLShapeUtil<T>;
    onRightPointHandle(shape: T, handle: Partial<T['handles']>, info: TLPointerInfo): TLShapeUtil<T>;
    onDoublePointHandle(shape: T, handle: Partial<T['handles']>, info: TLPointerInfo): TLShapeUtil<T>;
    onSessionComplete(shape: T): TLShapeUtil<T>;
    onBoundsReset(shape: T): TLShapeUtil<T>;
}
export interface IShapeTreeNode {
    shape: TLShape;
    children: IShapeTreeNode[];
    isEditing: boolean;
    isHovered: boolean;
    isSelected: boolean;
    isBinding: boolean;
    isDarkMode: boolean;
    isCurrentParent: boolean;
}
export declare enum MoveType {
    Backward = 0,
    Forward = 1,
    ToFront = 2,
    ToBack = 3
}
export declare enum AlignType {
    Top = 0,
    CenterVertical = 1,
    Bottom = 2,
    Left = 3,
    CenterHorizontal = 4,
    Right = 5
}
export declare enum StretchType {
    Horizontal = 0,
    Vertical = 1
}
export declare enum DistributeType {
    Horizontal = 0,
    Vertical = 1
}
export declare type MappedByType<T extends {
    type: string;
}> = {
    [P in T['type']]: T extends any ? (P extends T['type'] ? T : never) : never;
};
export declare type RequiredKeys<T> = {
    [K in keyof T]-?: Record<string, unknown> extends Pick<T, K> ? never : K;
}[keyof T];
